#ifndef SRC_GENERATOR_RF_H_
#define SRC_GENERATOR_RF_H_
#include <iostream>
#include <vector>
namespace mri {
  class GeneratorRF {
  public:
    template<class T> inline T pow2(const T value) {return value*value;}
    const double PI_=3.14159265358979323846;
    void InitApproximation();
    void InitTimer(unsigned long total_periods, double period_ms,
                   int samples_per_period);
    void InitVoxels(unsigned int voxel_num, double phase_init_,
                    double phase_range_, double T2s_scale);
    void EvaluateSignal(std::vector<double> &signal_rf);
    void EvaluateDemodulated(std::vector<double> &mag_sin,
                             std::vector<double> &mag_cos);
    void EvaluateApproximateDemodulated(std::vector<double> &mag_sin,
                                        std::vector<double> &mag_cos);
    void RandomizeVoxels();
    void SetAmplitudes(std::vector<double> amplitudes);
    void SetBase(const std::vector<double> &mag_sin,
                 const std::vector<double> &mag_cos);
    void SetNoiseAmplitude(double noise){noise_amplitude_ = noise;};
    void SetT2sDecaysNorm(std::vector<double> T2s_decays_norm);

    // amplitudes -> values[:size/2]
    // T2s_decays_norm -> values[size/2:]
    void SetVoxels(const std::vector<double> &values);
    void GetVoxels(std::vector<double> &values);
    
    double TryApproximateFit(const std::vector<double> &values);
    double TryFit(const std::vector<double> &values);
    const std::vector<double>& GetTimesteps(){return timesteps_;};

  private:
    double ApproxExp(int terms, double x);
    std::vector<double> base_mag_sin_, base_mag_cos_;
    std::vector<double> amplitudes_;
    std::vector<double> phases_;
    std::vector<double> T2s_decays_norm_;
    std::vector<int> exp_terms_;
    double T2s_scale_ = 0.01; // #ms # need to be 10ms
    /* double T2s_min_ = T2s_scale_/1000.0; */
    
    unsigned int voxel_num_ = 3;
    double phase_range_ = PI_/2;
    double phase_init_ = 0.0;
    
    double noise_amplitude_ = 1e-8;

    // B0=1.5T freq=64Mhz, period = 15.6 ns
    double period_ms_ = 15.6/1000/1000; //ms
    double omega_ = 2.0*PI_/period_ms_; 
    unsigned long total_periods_ = 1000;
    int samples_per_period_ = 10;
    unsigned long total_samples_ = total_periods_ * samples_per_period_;

    std::vector<double> timesteps_;
  };  // end of class GeneratorRF
}  // end of namespace mri
#endif  // SRC_GENERATOR_RF_H_
